@@ -1,4 +1,5 @@
 from fedbase.utils.data_loader import data_process, log
+from fedbase.utils.visualize import dimension_reduction
 from fedbase.nodes.node import node
 from fedbase.server.server import server_class
 import torch
@@ -9,6 +10,7 @@ import os
 import sys
 import inspect
 from functools import partial
+import numpy as np
 
 def run(dataset_splited, batch_size, K, num_nodes, model, objective, optimizer, global_rounds, local_steps, reg = None, device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')):
     # dt = data_process(dataset)
@@ -37,6 +39,8 @@ def run(dataset_splited, batch_size, K, num_nodes, model, objective, optimizer,
     server.distribute(nodes, list(range(num_nodes)))
 
     # train!
+    b_list = []
+    uu_list =[]
     for i in range(global_rounds):
         print('-------------------Global round %d start-------------------' % (i))
         # local update
@@ -47,6 +51,26 @@ def run(dataset_splited, batch_size, K, num_nodes, model, objective, optimizer,
                 nodes[j].local_update_steps(local_steps, partial(nodes[j].train_single_step_fedprox, reg_model = server.aggregate(nodes, list(range(num_nodes))), lam= reg))
         # server clustering
         server.weighted_clustering(nodes, list(range(num_nodes)), K)
+
+        # # tsne or pca plot
+        # # if i == global_rounds-1:
+        # if i == i :
+        #     cluster_data = torch.nn.utils.parameters_to_vector(nodes[0].model.parameters()).cpu().detach().numpy()[-1000:]
+        #     for i in range(1, num_nodes):
+        #         cluster_data = np.concatenate((cluster_data, torch.nn.utils.parameters_to_vector(nodes[i].model.parameters()).cpu().detach().numpy()[-1000:]), axis = 0)
+        #     cluster_data = np.reshape(cluster_data, (num_nodes,int(len(cluster_data)/num_nodes)))
+        #     cluster_label = server.clustering['label'][-1]
+        #     # cluster_label = np.repeat(range(10),20)
+        #     dimension_reduction(cluster_data, cluster_label, method= 'tsne')
+        # plot B
+        # print(server.calculate_B(nodes, range(20)))
+        # B_list, u_list = server.calculate_B(nodes, range(20))
+        # b_list.append(max(B_list))
+        # uu_list.append(max(u_list))
+        # print(b_list, uu_list)
+        for k in range(num_nodes):
+            nodes[k].grads = []
+        # print(a)
         # server aggregation and distribution by cluster
         for i in range(K):
             server.aggregate(nodes, [j for j in list(range(num_nodes)) if nodes[j].label==i])
@@ -57,4 +81,8 @@ def run(dataset_splited, batch_size, K, num_nodes, model, objective, optimizer,
         server.acc(nodes, list(range(num_nodes)))
     
     # log
-    log(os.path.basename(__file__)[:-3] + '_' + str(K) + '_' + str(reg) + '_' + split_para, nodes, server)
\ No newline at end of file
+    if reg:
+        log(os.path.basename(__file__)[:-3] + '_' + str(K)  + '_' + split_para, nodes, server)
+    else:
+        log(os.path.basename(__file__)[:-3] + '_' + str(K) + '_' + str(reg) + '_' + split_para, nodes, server)
+    
\ No newline at end of file
